CON

  _clkmode      = xtal1 + pll8x
  _xinfreq      = 10_000_000

  LEFT_MOTOR    = 0
  RIGHT_MOTOR   = 1

  LEFT_NULL     = 1500
  RIGHT_NULL    = 1500

  #0,  C,D,E,F,G,A,B,CC,Z
  
  Et = $10
  Qu = $20
  DQ = $30
  Hl = $40
  DH = $60
  Wh = $80
  DW = $C0

  #Et+C, EC,ED,EE,EF,EG,EA,EB,ECC,EZ
  #Qu+C, QC,QD,QE,QF,QG,QA,QB,QCC,QZ
  #DQ+C, DQC,DQD,DQE,DQF,DQG,DQA,DQB,DQCC,DQZ
  #Hl+C, HC,HD,HE,HF,HG,HA,HB,HCC,HZ
  #DH+C, DHC,DHD,DHE,DHF,DHG,DHA,DHB,DHCC,DHZ
  #Wh+C, FC,WD,WE,WF,WG,WA,WB,WCC,FZ
  #DW+C, DWC,DWD,DWE,DWF,DWG,DWA,DWB,DWCC,DWZ

  END = 0

  NTUNES = 9

VAR

  long left_pulse, right_pulse

OBJ

  ab    :       "ActivityBoard"

PUB start | base, tune, index, pindex

  pindex := -1
  base := @scale - tunes[0]
  repeat
    repeat until ((index := ab.random(NTUNES)) <> pindex)
    play_tune(tunes[index] + base)
    pindex := index
    waitcnt(cnt + clkfreq * 5)

PUB play_tune(tune) | tempo, i, note, rotate, up, down, duration

  tempo := byte[tune++]
  i := 0
  repeat while duration := (note := byte[tune][i++]) >> 4
    duration := 1500 * duration / tempo
    note &= $0f
    rotate := pulses[note]
    up := ups[sides[note]]
    down := downs[sides[note]]
    pulse_motors(rotate, up, 10)
    if (note <> Z)
      pulse_motors(rotate, down, 1)
    pulse_motors(rotate, up, (duration - 10) #> 1)

PUB ramp_motors(left, right, n) | i

  repeat i from 0 to n
    pulse_motors((left_pulse * (n - i) + left * i) / n, (right_pulse * (n - i) + right * i) / n, 1)
  left_pulse := left
  right_pulse := right
     
PUB pulse_motors(left, right, how_many_times)

  repeat how_many_times
    ab.pulse_usec(LEFT_MOTOR, left)
    ab.pulse_usec(RIGHT_MOTOR, right)
    ab.delay_msec(20 - (left + right) / 1000)

DAT

tunes         word      @scale,@saints,@lullaby,@beer,@westminster,@twinkle,@betsy
              word      @ode_to_joy,@frere_jaques

scale         byte      240,QC,QD,QE,QF,QG,QA,QB,WCC,QC,QE,QG,QCC,QG,QE,QC,END

saints        byte      240,QC,QE,QF,Wh+QG,QC,QE,QF,Wh+QG,QC,QE,QF,hG,hE,hC,hE,Wh+QD
              byte      QE,QE,QD,hC,HZ,QC,HE,QG,QG,QG,Wh+QF,QE,QF,hG,hE,hC,hD,FC,END

lullaby       byte      180,qe,qe,dhg,qe,qe,dhg,qe,qg,hcc,dhb,qa,ha,hg,qd,qe,hf,hd,qd,qe,wf
              byte      qd,qf,qb,qa,hg,hb,wcc,qc,qc,wcc,qa,qf,wg,qe,qc,hf,hg,ha,wg,qc,qc
              byte      wcc,qa,qf,wg,qe,qc,hf,he,hd,fc,end

beer          byte      240,qf,qf,qf,qc,qc,qc,qf,qf,qf,dhf,qg,qg,qg,qd,qd,qd,dwg
              byte      he,qe,dhe,qe,qe,qe,dhe,qc,qc,qc,qd,qd,qe,qf,qf,qf,dhf
              byte      qf,qf,qf,qc,qc,qc,qf,qf,qf,dhf,qg,qg,qg,qd,qd,qd,dwg
              byte      he,qe,dhe,qe,qe,qe,dhe,qc,qc,qc,qd,qd,qe,qf,qf,qf,dhf,end

westminster   byte      120,qa,qf,qg,hc,qc,qg,qa,hf,qa,qg,qf,hc,qc,qg,qa,hf,end

twinkle       byte      240,qc,qc,qg,qg,qa,qa,hg,qf,qf,qe,qe,qd,qd,hc
              byte      qg,qg,qf,qf,qe,qe,hd,qg,qg,qf,qf,qe,qe,hd
              byte      qc,qc,qg,qg,qa,qa,hg,qf,qf,qe,qe,qd,qd,hc,end

betsy         byte      240,qc,qc,qe,qg,qg,qf,qd,qd,qc,qc,qz
              byte      qc,qc,qe,qg,qcc,qcc,qcc,qb,qg,qg,qg,qz
              byte      qg,qcc,qcc,qcc,qb,qg,qe,qf,qg,qa,qg,qz
              byte      qc,qe,qe,qe,qg,qf,qd,qd,qc,qc,qc,qz,end

ode_to_joy    byte      120,qe,qe,qf,qg,qg,qf,qe,qd,qc,qc,qd,qe,dqe,ed,dhd
              byte      qe,qe,qf,qg,qg,qf,qe,qd,qc,qc,qd,qe,dqd,ec,hc
              byte      qd,qd,qe,qc,qd,ee,ef,qe,qc,qd,ee,ef,qe,qd,qc,qd,hg
              byte      qe,qe,qf,qg,qg,qf,qe,qd,qc,qc,qd,qe,dqd,ec,hc,end

frere_jaques  byte      120,qc,qd,qe,qc,qc,qd,qe,qc,qe,qf,hg,qe,qf,hg,eg
              byte      ea,eg,ef,qe,qc,eg,ea,eg,ef,qe,qc,qc,qg,hc,qc,qg,hc,end

pulses        long      1900, 1240, 1900, 1240, 1670, 1450, 1660, 1450  
sides         long         0,    1,    1,    0,    0,    0,    1,    1

ups           long      1600, 1550
downs         long      1680, 1460      
               